/**********************************************************************
 * Copyright (c) 2003, 2012 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * $Id: XMLclassDefLoader.java,v 1.17 2007-02-19 22:15:01 slavescu Exp $
 *
 * Contributors:
 * IBM - Initial API and implementation
 * 
 * $Id: XMLclassDefLoader.java,v 1.17 2007-02-19 22:15:01 slavescu Exp $
 **********************************************************************/
package org.eclipse.hyades.loaders.trace;

import org.eclipse.hyades.loaders.hierarchy.Constants;
import org.eclipse.hyades.loaders.util.HierarchyContext;
import org.eclipse.hyades.loaders.util.LoadersUtils;
import org.eclipse.hyades.loaders.util.LookupServiceExtensions;
import org.eclipse.hyades.models.hierarchy.TRCTIProfilerCollectionMode;
import org.eclipse.hyades.models.trace.TRCArrayClass;
import org.eclipse.hyades.models.trace.TRCClass;
import org.eclipse.hyades.models.trace.TRCFullTraceObject;
import org.eclipse.hyades.models.trace.TraceFactory;
import org.eclipse.hyades.models.trace.TracePackage;
import org.eclipse.hyades.models.trace.impl.TRCClassImpl;
import org.eclipse.hyades.models.trace.impl.TRCFullHeapObjectImpl;
import org.eclipse.hyades.models.trace.impl.TRCFullTraceObjectImpl;
import org.eclipse.hyades.models.trace.impl.TRCHeapObjectImpl;

/**
 * @author slavescu
 *
 */
public class XMLclassDefLoader extends TraceXMLFragmentLoader {
	//~ Static fields/initializers -----------------------------------------------------------------

	protected static final String NUM_STATIC_FIELDS = "numStaticFields";

	protected static final String NUM_METHODS = "numMethods";

	protected static final String NUM_INTERFACES = "numInterfaces";

	protected static final String NUM_INSTANCES_FIELDS = "numInstanceFields";

	protected static final String ACCESS = "access";

	protected static final String CLASS_ID = "classId";

	protected static final String IS_CLASS_CLASS = "isClassClass";

	//~ Instance fields ----------------------------------------------------------------------------

	protected String access;

	protected String name;

	protected long classId;

	protected int numInstanceFields;

	protected int numInterfaces;

	protected int numMethods;

	protected int numStaticFields;

	protected boolean java_lang_class;

	protected boolean isClassClass;

	//~ Methods ------------------------------------------------------------------------------------

	public void addAttribute(String name, String value) {
		switch (LoadersUtils.getHashCode(name)) {
		case TraceConstants.CLASS_ID_int:
			classId = Long.parseLong(value);

			break;

		case TraceConstants.NAME_int:
			if (currentAnnotation != null)
				currentAnnotation.setName(value);
			else
				this.name = value;

			break;

		case TraceConstants.ACCESS_int:
			access = value;

			break;

		case TraceConstants.NUM_INSTANCES_FIELDS_int:
			numInstanceFields = Integer.parseInt(value);

			break;

		case TraceConstants.NUM_INTERFACES_int:
			numInterfaces = Integer.parseInt(value);

			break;

		case TraceConstants.NUM_METHODS_int:
			numMethods = Integer.parseInt(value);

			break;

		case TraceConstants.NUM_STATIC_FIELDS_int:
			numStaticFields = Integer.parseInt(value);

			break;

		default:
			super.addAttribute(name, value);

			break;
		}
	}

	public void addYourselfInContext() {
		if (sourceName == null) {
			sourceName = UNKNOWN;
		}

		if (name == null || name.length() == 0) {
			name = UNKNOWN;
		}
		if (context.isFilterOn()) {
			if (context.getFilterEngine().isFiltered(TracePackage.eINSTANCE.getTRCThread_Name(), LoadersUtils.getLookUpKey(threadIdRef))) {
				return;
			}
			if (context.getFilterEngine().isFiltered(TracePackage.eINSTANCE.getTRCPackage_Name(), LoadersUtils.getLookUpKey(TraceUtils.packageName(name)), TraceUtils.packageName(name))) {
				context.getFilterEngine().addExcludedId(TracePackage.eINSTANCE.getTRCClass_Name(), LoadersUtils.getLookUpKey(classId));
				return;
			}
			if (context.getFilterEngine().isFiltered(TracePackage.eINSTANCE.getTRCClass_Name(), LoadersUtils.getLookUpKey(classId), TraceUtils.className(name))) {
				return;
			}
		}

		createTRCType();
	}

	public void initialize(HierarchyContext context, String name) {
		super.initialize(context, name);
		classId = 0;
		sourceName = null;
		name = null;
		access = null;
		numStaticFields = 0;
		numMethods = 0;
		numInstanceFields = 0;
		language = null;
		annotations.clear();
		currentAnnotationValueEntryActive=false;
		isClassClass=false;
	}

	protected void processEF(int step) {
		super.processEF(step);
		processES(step);
	}

	/**
	 *
	 */
	protected void processES(int step) {
		super.processES(step);

		switch (step) {
		case ProcessSteps.GET_CLASS_OBJECT:
			if(createClassObjects)
			{
				/* Look for the class object in the model if we have been given an object Id*/
				if (objIdRef != 0) {
					theObject = getClassObjectByIdRef(objIdRef, TRCFullTraceObjectImpl.class);
				} else {
					theObject = getClassObjectByIdRef(-classId, TRCFullTraceObjectImpl.class);
				}
	
				if (theObject == null) {
					// now add a class object
					theObject = TraceFactory.eINSTANCE.createTRCFullTraceObject();
	
					if (objIdRef == 0) {
						theObject.setId(-classId);
					} else {
						theObject.setId(objIdRef);
						updateTRCFullTraceObjectFromVOI(objIdRef);
					}
	
					theObject.setProcess(theProcess);
				}
	
				if (!theObject.eIsSet(TracePackage.eINSTANCE.getTRCObject_Size()))
					theObject.setSize(theClass.getSize());
				theClass.getClassObjects().add(theObject);
	
				//adds the class object to the Class class
				TraceUtils.addClassObjectToClassClass(context, theProcess, theObject);
			}
			break;

		default:
			break;
		}
	}

	/**
	 *
	 */
	protected void processHF(int step) {
		super.processHF(step);
		processHS(step);
	}

	@SuppressWarnings("rawtypes")
	protected void processHS(int step) {
		super.processHS(step);

		switch (step) {
		case ProcessSteps.GET_CLASS_OBJECT:
			theObject = null;
			Class clazz = (TraceUtils.isBooleanOptionEnabled(context, Constants.MULTIPLE_HEAP_DUMPS) ? TRCFullHeapObjectImpl.class : TRCHeapObjectImpl.class);
			/* Look for the class object in the model if we have been given an object Id*/
			if (objIdRef != 0) {
				theObject = getClassObjectByIdRef(objIdRef, clazz);
			} else {
				theObject = getClassObjectByIdRef(-classId, clazz);
			}
			if (theObject == null) {
				// now add a class object
				if (TraceUtils.isBooleanOptionEnabled(context, Constants.MULTIPLE_HEAP_DUMPS))
					theObject = TraceFactory.eINSTANCE.createTRCFullHeapObject();
				else
					theObject = TraceFactory.eINSTANCE.createTRCHeapObject();
				if (objIdRef == 0) {
					theObject.setId(-classId);
				} else {
					theObject.setId(objIdRef);
				}

				theObject.setProcess(theProcess);
			}

			theClass.getClassObjects().add(theObject);

			//adds the class object to the Class class
			TraceUtils.addClassObjectToClassClass(context, theProcess, theObject);

			break;

		default:
			break;
		}
	}

	protected void updateTRCFullTraceObjectFromVOI(long oId) {
		VirtualObjectInfo objectInfo = (VirtualObjectInfo) LookupServiceExtensions.getInstance().locate(context, VirtualObjectInfo.class, oId);

		if (objectInfo != null) {
			TRCFullTraceObject object = (TRCFullTraceObject) theObject;

			object.setCreateTime(objectInfo.createTime);
			object.setSize(objectInfo.size);
			TraceUtils.removeVirtualObject(context, objIdRef);
		}
	}

	private void createTRCType() {
		theProcess = getProcess();
		java_lang_class = false;

		int hash = LoadersUtils.getHashCode(name);

		if (isClassClass || ((theProcess.getLanguages().isEmpty() || theProcess.getLanguages().indexOf("Java")!=-1) && (hash == TraceConstants.CLASS_CLASS1_int) || (hash == TraceConstants.CLASS_CLASS2_int))) //if class Class
		{
			// last update wins - update the class Class info
			theClass = theProcess.getClassClass();

			if (theClass != null) {
				if (theClass.getId() != classId) {
					LookupServiceExtensions.getInstance().deregister(context, TRCClassImpl.class, theClass.getId());
					theClass.setId(classId);
					LookupServiceExtensions.getInstance().register(context, theClass);
				}
			} else {
				theClass = (TRCClass) LookupServiceExtensions.getInstance().locate(context, TRCClassImpl.class, classId);
				java_lang_class = true;
			}
		} else {
			theClass = (TRCClass) LookupServiceExtensions.getInstance().locate(context, TRCClassImpl.class, classId);
		}
		
		String className = TraceUtils.className(name);
		
		if (theClass == null) {
			String packageName = TraceUtils.packageName(name);
			theClass = TraceUtils.findClass(theProcess, className, packageName);
			if(theClass==null)
			{
				if (name.charAt(0) == '[') {
					theClass = TraceFactory.eINSTANCE.createTRCArrayClass();
					
	                TRCClass elementClass = TraceUtils.findClass(theProcess, className.substring(0,className.lastIndexOf("[]")),packageName);
	
	                if (elementClass != null) {
	                    ((TRCArrayClass) theClass).setElementType(elementClass);
	                }
				} else {
					theClass = TraceFactory.eINSTANCE.createTRCClass();
				}
			}
		}

		// last update wins
		theClass.setName(className);
		theClass.setOrigFullName(name);
		theClass.setId(classId);
		theClass.setLoadTime(createDeltaTime());
		if(theClass.getPackage()!=null)
		{
			//reset to trigger registration with the lookup service
			theClass.setPackage(null);
		}
	
		TraceUtils.addClassToPackage(context, theClass, name, theProcess);

		TraceUtils.setSourceInfo(context, theProcess, theClass, sourceName, language);

		addLanguageIfRequired(theProcess.getLanguages());
		addAnnotationsIfRequired(theClass);

		if ((theProcess.getClassClass() == null) && java_lang_class) // if class Class not set
		{
			theProcess.setClassClass(theClass);

			//	else do nothing because a fake one will be created automaticaly
		}

		// Thread profiling should NOT set the class' loadedBy value, while all other profiling types should
		boolean setLoadedBy = true;
		if(theProcess != null && theProcess.getAgent() != null && theProcess.getAgent().getAgentProxy() != null) {
			TRCTIProfilerCollectionMode profMode = theProcess.getAgent().getAgentProxy().getTiCollectionMode();
			if(profMode != null && 
					(profMode == TRCTIProfilerCollectionMode.THREAD_CONTENTION ||
					profMode == TRCTIProfilerCollectionMode.THREAD_NO_CONTENTION)) {
				setLoadedBy = false;
			}
		}
		
		dispatchProcessMode(ProcessSteps.GET_CLASS_OBJECT);
		
		if(setLoadedBy) {
			// Exec and heap should still set this value, thread should not (calling getThreadIdByRef causes an 'unknown' thread to appear in the thread list, when thread profiling.)
			theThread = getThreadByIdRef(theProcess);
			theClass.setLoadedBy(theThread);
		}
		


	}

	//    private TRCClass findClass(TRCProcess p, String name) {
	//        return null;
	//    }
}
