/**********************************************************************
 * Copyright (c) 2003, 2011 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * $Id: XMLruntimeInitDoneLoader.java,v 1.3 2006-02-08 19:56:55 slavescu Exp $
 *
 * Contributors:
 * IBM - Initial API and implementation
 * 
 * $Id: XMLruntimeInitDoneLoader.java,v 1.3 2006-02-08 19:56:55 slavescu Exp $
 **********************************************************************/
package org.eclipse.hyades.loaders.trace;

import org.eclipse.hyades.loaders.hierarchy.IgnoredXMLFragmentLoader;
import org.eclipse.hyades.loaders.util.HierarchyContext;
import org.eclipse.hyades.models.hierarchy.HierarchyFactory;
import org.eclipse.hyades.models.hierarchy.TRCAgentProxy;
import org.eclipse.hyades.models.hierarchy.TRCConfiguration;
import org.eclipse.hyades.models.hierarchy.TRCOption;
import org.eclipse.hyades.models.hierarchy.TRCTIProfilerCollectionMode;

public class XMLruntimeInitDoneLoader extends IgnoredXMLFragmentLoader {

	/** These values references plugin.xml values in various workbench-wide plugins */
	public static final String EXECUTION_ANALYSIS_TYPE = "org.eclipse.tptp.analysisType.jvmti.execution";
	public static final String HEAP_ANALYSIS_TYPE = "org.eclipse.tptp.analysisType.jvmti.heap";
	public static final String THREAD_ANALYSIS_TYPE = "org.eclipse.tptp.analysisType.jvmti.thread";

	/** This value references a value in the plugin.xml of org.eclipse.hyades.trace.ui */
	public static final String ANALYSIS_TYPE_ASSOCIATION = "org.eclipse.hyades.trace.ui.ANALYSIS_TYPE_ASSOCIATION";

	
    public void initialize(HierarchyContext context, String name) {
        super.initialize(context, name);
    }
	
	@Override
	public void addYourselfInContext() {
		utilSetAssociation(context.getAgentProxy());
	}
	
	/** Examine the JVMTI type, and -- if needed -- set the association value based on the JVMTI type*/
	public static void utilSetAssociation(TRCAgentProxy proxy) {
	
		
		// Locate the JVMTI configuration
		TRCConfiguration config = null;
		if(proxy.getConfigurations() != null) {
			for(TRCConfiguration c : proxy.getConfigurations()) {
				if(c.getName() != null && c.getName().equalsIgnoreCase("__JVMPI__")) {
					config = c;
				}
			}
		}
		
		if(config != null && config.getOptions() != null) {
			
			// Look through the configuration options for the analysis type association, which should exist
			boolean analysisTypeFound = false;
			for(TRCOption o : config.getOptions()) {
				if(o.getKey().startsWith(ANALYSIS_TYPE_ASSOCIATION+"(")) {
					analysisTypeFound = true;
					break;
				}
			}
			
			// If it doesn't exist, using the TI profiling type, and set this value (75976)
			if(!analysisTypeFound) {

				TRCTIProfilerCollectionMode tiMode = proxy.getTiCollectionMode();
				if(tiMode != null) {
					String analysisType = null;
					
					if(tiMode == TRCTIProfilerCollectionMode.EXEC_FLOW || 
							tiMode == TRCTIProfilerCollectionMode.EXEC_STATS) {
						analysisType = EXECUTION_ANALYSIS_TYPE;
					}
					
					if(tiMode == TRCTIProfilerCollectionMode.HEAP_ALLOC_SITES || 
							tiMode == TRCTIProfilerCollectionMode.HEAP_NO_ALLOC_SITES) {
						
						analysisType = HEAP_ANALYSIS_TYPE;
					}
					
					if(tiMode == TRCTIProfilerCollectionMode.THREAD_CONTENTION || 
							tiMode == TRCTIProfilerCollectionMode.THREAD_NO_CONTENTION) {
						
						analysisType = THREAD_ANALYSIS_TYPE;
												
					}
					
					if(analysisType != null) {
						TRCOption option = HierarchyFactory.eINSTANCE.createTRCOption();
						option.setKey(ANALYSIS_TYPE_ASSOCIATION+"("+analysisType+")");
						option.setValue(analysisType);				
						config.getOptions().add(option);
					}
					
				}
				
			}
			
		}
	}
}
