/*******************************************************************************
 * Copyright (c) 2005, 2010 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * $Id: DatapoolTableUtil.java,v 1.11 2010-02-03 13:52:56 paules Exp $
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.hyades.test.ui.datapool.internal.control;

import org.eclipse.hyades.edit.datapool.IDatapoolCell;
import org.eclipse.hyades.edit.datapool.IDatapoolEquivalenceClass;
import org.eclipse.hyades.edit.datapool.IDatapoolRecord;
import org.eclipse.hyades.edit.datapool.IDatapoolVariable;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;

/**
 * <p>DatapoolTableUtil.java</p>
 * 
 * 
 * @author  Peter Sun
 * @author  Paul E. Slauenwhite
 * @version February 3, 2010
 * @since   April 12, 2008
 */
public class DatapoolTableUtil {

	public static final String TAG_DATA = "data"; //$NON-NLS-1$
	public static final String TAG_RECORD = "record"; //$NON-NLS-1$
	public static final String TAG_VARIABLE = "variable"; //$NON-NLS-1$
	public static final String TAG_EQUIVALENCE_CLASS = "equivalenceClass"; //$NON-NLS-1$

	private DatapoolTable dpTable = null;
	private Table table = null;
	
	public DatapoolTableUtil(DatapoolTable table)
	{
		this.dpTable = table;
		this.table = (Table)dpTable.getViewer().getControl();
	}

	/**
	 * <p>Resolves the number of columns in the datapool table.</p>
	 * 
	 * @return Number of columns in the datapool table
	 */
	public int getColumnCount(){
		return (table.getColumnCount());
	}

	// Assumes column has been added
	public void insertColumn(TableColumn tableColumn, int insertionIndex)
	{
		int sourceIndex = findColumnIndex(tableColumn);
		// Add to rowData
		for(int i = 0; i < table.getItemCount(); i++)
		{
			TableItem tableItem = table.getItem(i);	
			IDatapoolCell[] rowData = (IDatapoolCell[])tableItem.getData(TAG_DATA);
			IDatapoolCell[] newRowData = new IDatapoolCell[table.getColumnCount() - 1];
			String[] newRowContext = new String[table.getColumnCount()];
			newRowContext[0] = tableItem.getText(0);
			for(int j = 0; j < newRowData.length; j++)
			{
				if(j < insertionIndex)
				{
					newRowData[j] = rowData[j];
					newRowContext[j+1] = tableItem.getText(j+1);
				}
				if(j > insertionIndex)
				{
					try
					{
						newRowData[j] = rowData[j - 1];
						newRowContext[j] = tableItem.getText(j);
					}
					catch(IndexOutOfBoundsException e)
					{
					}
				}
				if(j == insertionIndex)
				{
					IDatapoolRecord record = (IDatapoolRecord)tableItem.getData(TAG_RECORD);
					newRowData[j] = (IDatapoolCell)record.getCell(j);
				}								
			}
			tableItem.setData(TAG_DATA, newRowData);
			tableItem.setText(newRowContext);
		}
		
		if(sourceIndex >= 0)
		{
			if(sourceIndex != insertionIndex + 1)
			{
				moveColumn(sourceIndex, insertionIndex + 1);
				TableColumn sourceColumn = table.getColumn(sourceIndex);
				TableColumn insertedColumn = table.getColumn(insertionIndex + 1);
				sourceColumn.setWidth(100);
				sourceColumn.setResizable(true);
				insertedColumn.setWidth(100);
				insertedColumn.setResizable(true);
			}
		}
		else
			throw new IllegalArgumentException();
	}

	/**
	 * <p>Deletes the column in the datapool table at the column index.</p>
	 * 
	 * @param columnIndex The column index.
	 */
	public void deleteColumn(int columnIndex){
		
		TableColumn tableColumn = null;
		
		if((columnIndex + 1 == getColumnCount()) || (columnIndex == 0)){
			tableColumn = table.getColumn(columnIndex);
		}
		else{
			
			moveColumn(columnIndex, (getColumnCount() - 1));
			
			tableColumn = table.getColumn(getColumnCount() - 1);
		}	
		
		if(tableColumn != null){	
			tableColumn.dispose();			
		}
	}
	
	private int findColumnIndex(TableColumn column)
	{
		for(int i = 0; i < table.getColumnCount(); i++)
		{
			if(table.getColumn(i).equals(column))
				return i;
		}
		return -1;
	}
	
	public void swapColumn(int sourceColumnIndex, int targetColumnIndex)
	{
		TableColumn sourceColumn = table.getColumn(sourceColumnIndex);
		String sourceName = sourceColumn.getText();
		IDatapoolVariable sourceVariable = (IDatapoolVariable)sourceColumn.getData(TAG_VARIABLE);
		Image sourceImage = sourceColumn.getImage();
		
		TableColumn targetColumn = table.getColumn(targetColumnIndex);		
		String targetName = targetColumn.getText();
		IDatapoolVariable targetVariable = (IDatapoolVariable)targetColumn.getData(TAG_VARIABLE);
		Image targetImage = targetColumn.getImage();
		
		sourceColumn.setText(targetName);
		sourceColumn.setData(TAG_VARIABLE, targetVariable);
		sourceColumn.setImage(targetImage);
		
		targetColumn.setText(sourceName);		
		targetColumn.setData(TAG_VARIABLE, sourceVariable);
		targetColumn.setImage(sourceImage);
		
		for(int i = 0; i < table.getItemCount(); i++)
		{
			TableItem tableItem = table.getItem(i);	
			
			IDatapoolCell[] rowData = (IDatapoolCell[])tableItem.getData(TAG_DATA);
			String[] newRowContext = new String[table.getColumnCount()];
			newRowContext[0] = tableItem.getText(0);
			for(int j = 0; j < rowData.length; j++)
				newRowContext[j+1] = tableItem.getText(j+1);
			newRowContext[sourceColumnIndex] = tableItem.getText(targetColumnIndex);
			newRowContext[targetColumnIndex] = tableItem.getText(sourceColumnIndex);
			IDatapoolCell sourceCell = rowData[sourceColumnIndex-1];
			IDatapoolCell targetCell = rowData[targetColumnIndex-1];
			rowData[sourceColumnIndex-1] = targetCell;
			rowData[targetColumnIndex-1] = sourceCell;
			tableItem.setData(TAG_DATA, rowData);
			tableItem.setText(newRowContext);
		}			
	}

	public void moveColumn(int sourceColumnIndex, int targetColumnIndex)
	{
		int numToSwap = 0;
		if(sourceColumnIndex < targetColumnIndex)
			numToSwap = targetColumnIndex - sourceColumnIndex;
		else
			numToSwap = sourceColumnIndex - targetColumnIndex;

		TableColumn sourceColumn = table.getColumn(sourceColumnIndex);
		String sourceName = sourceColumn.getText();
		IDatapoolVariable sourceVariable = (IDatapoolVariable)sourceColumn.getData(TAG_VARIABLE);
		Image sourceImage = sourceColumn.getImage();
		
		for(int i = 0; i < numToSwap; i++)
		{
			TableColumn shiftColumn = null;
			TableColumn targetColumn = null;

			if(sourceColumnIndex < targetColumnIndex)
			{
				shiftColumn = table.getColumn(sourceColumnIndex + i + 1);
				targetColumn = table.getColumn(sourceColumnIndex + i);
			}
			else
			{
				shiftColumn = table.getColumn(sourceColumnIndex - i - 1);
				targetColumn = table.getColumn(sourceColumnIndex - i);
			}
			
			String shiftName = shiftColumn.getText();
			IDatapoolVariable shiftVariable = (IDatapoolVariable)shiftColumn.getData(TAG_VARIABLE);
			Image shiftImage = shiftColumn.getImage();
			
			targetColumn.setText(shiftName);
			targetColumn.setData(TAG_VARIABLE, shiftVariable);
			targetColumn.setImage(shiftImage);
		}
		
		TableColumn targetColumn = table.getColumn(targetColumnIndex);		
		targetColumn.setText(sourceName);		
		targetColumn.setData(TAG_VARIABLE, sourceVariable);
		targetColumn.setImage(sourceImage);
		
		for(int i = 0; i < table.getItemCount(); i++)
		{
			TableItem tableItem = table.getItem(i);	
			
			IDatapoolCell[] rowData = (IDatapoolCell[])tableItem.getData(TAG_DATA);
			IDatapoolCell sourceCell = rowData[sourceColumnIndex - 1];
			String sourceText = tableItem.getText(sourceColumnIndex);
			
			for(int j = 0; j < numToSwap; j++)
			{
				if(sourceColumnIndex < targetColumnIndex)
				{
					IDatapoolCell shiftCell = rowData[sourceColumnIndex + j];
					String shiftText = tableItem.getText(sourceColumnIndex + j + 1);
					
					rowData[sourceColumnIndex + j - 1] = shiftCell;
					tableItem.setText(sourceColumnIndex + j, shiftText);
				}
				else
				{
					IDatapoolCell shiftCell = rowData[sourceColumnIndex - j - 2];
					String shiftText = tableItem.getText(sourceColumnIndex - j - 1);
				
					rowData[sourceColumnIndex - j - 1] = shiftCell;
					tableItem.setText(sourceColumnIndex - j, shiftText);														
				}
			}
			
			tableItem.setText(targetColumnIndex, sourceText);
			rowData[targetColumnIndex - 1] = sourceCell;

			tableItem.setData(TAG_DATA, rowData);
		}					
	}
	
	public void swapRow(int sourceRowIndex, int targetRowIndex)
	{
		TableItem sourceItem = table.getItem(sourceRowIndex);
		String[] sourceRowContext = new String[table.getColumnCount()];
		for(int i = 0; i < table.getColumnCount(); i++)
		{
			sourceRowContext[i] = sourceItem.getText(i);
		}	
		IDatapoolCell[] sourceRowData = (IDatapoolCell[])sourceItem.getData(TAG_DATA);
		IDatapoolRecord sourceRecord = (IDatapoolRecord)sourceItem.getData(TAG_RECORD);
		IDatapoolEquivalenceClass sourceEquivalenceClass = (IDatapoolEquivalenceClass)sourceItem.getData(TAG_EQUIVALENCE_CLASS);
				
		TableItem targetItem = table.getItem(targetRowIndex);
		String[] targetRowContext = new String[table.getColumnCount()];
		for(int i = 0; i < table.getColumnCount(); i++)
		{
			targetRowContext[i] = targetItem.getText(i);
		}
		IDatapoolCell[] targetRowData = (IDatapoolCell[])targetItem.getData(TAG_DATA);
		IDatapoolRecord targetRecord = (IDatapoolRecord)targetItem.getData(TAG_RECORD);
		IDatapoolEquivalenceClass targetEquivalenceClass = (IDatapoolEquivalenceClass)targetItem.getData(TAG_EQUIVALENCE_CLASS);

		sourceItem.setText(targetRowContext);
		sourceItem.setData(TAG_DATA, targetRowData);
		sourceItem.setData(TAG_RECORD, targetRecord);
		sourceItem.setData(TAG_EQUIVALENCE_CLASS, targetEquivalenceClass);
		
		targetItem.setText(sourceRowContext);
		targetItem.setData(TAG_DATA, sourceRowData);
		targetItem.setData(TAG_RECORD, sourceRecord);
		targetItem.setData(TAG_EQUIVALENCE_CLASS, sourceEquivalenceClass);
	}
	
	public void moveRow(int sourceRowIndex, int targetRowIndex)
	{
		int numToSwap = 0;
		if(sourceRowIndex < targetRowIndex)
			numToSwap = targetRowIndex - sourceRowIndex;
		else
			numToSwap = sourceRowIndex - targetRowIndex;
		
		TableItem sourceItem = table.getItem(sourceRowIndex);
		String[] sourceRowContext = new String[table.getColumnCount()];
		for(int i = 0; i < table.getColumnCount(); i++)
		{
			sourceRowContext[i] = sourceItem.getText(i);
		}	
		IDatapoolCell[] sourceRowData = (IDatapoolCell[])sourceItem.getData(TAG_DATA);
		IDatapoolRecord sourceRecord = (IDatapoolRecord)sourceItem.getData(TAG_RECORD);
		IDatapoolEquivalenceClass sourceEquivalenceClass = (IDatapoolEquivalenceClass)sourceItem.getData(TAG_EQUIVALENCE_CLASS);

		for(int i = 0; i < numToSwap; i++)
		{
			TableItem shiftItem = null;
			TableItem targetItem = null;

			if(sourceRowIndex < targetRowIndex)
			{
				shiftItem = table.getItem(sourceRowIndex + i + 1);
				targetItem = table.getItem(sourceRowIndex + i);
			}
			else
			{
				shiftItem = table.getItem(sourceRowIndex - i - 1);
				targetItem = table.getItem(sourceRowIndex - i);
			}
			
			String[] shiftRowContext = new String[table.getColumnCount()];
			for(int j = 0; j < table.getColumnCount(); j++)
			{
				shiftRowContext[j] = shiftItem.getText(j);
			}	
			IDatapoolCell[] shiftRowData = (IDatapoolCell[])shiftItem.getData(TAG_DATA);
			IDatapoolRecord shiftRecord = (IDatapoolRecord)shiftItem.getData(TAG_RECORD);
			IDatapoolEquivalenceClass shiftEquivalenceClass = (IDatapoolEquivalenceClass)shiftItem.getData(TAG_EQUIVALENCE_CLASS);
			
			targetItem.setText(shiftRowContext);
			targetItem.setData(TAG_DATA, shiftRowData);
			targetItem.setData(TAG_RECORD, shiftRecord);
			targetItem.setData(TAG_EQUIVALENCE_CLASS, shiftEquivalenceClass);
		}
		
		TableItem targetItem = table.getItem(targetRowIndex);
		targetItem.setText(sourceRowContext);
		targetItem.setData(TAG_DATA, sourceRowData);
		targetItem.setData(TAG_RECORD, sourceRecord);
		targetItem.setData(TAG_EQUIVALENCE_CLASS, sourceEquivalenceClass);
	}
}
