/*******************************************************************************
 * Copyright (c) 2005, 2008 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * $Id: TestComponentsForm.java,v 1.6 2008-01-21 17:19:20 paules Exp $
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.hyades.test.ui.internal.editor.form;

import org.eclipse.hyades.models.common.testprofile.Common_TestprofilePackage;
import org.eclipse.hyades.models.common.testprofile.TPFTestSuite;
import org.eclipse.hyades.test.ui.UiPlugin;
import org.eclipse.hyades.test.ui.editor.extension.TestSuiteEditorExtension;
import org.eclipse.hyades.test.ui.editor.form.util.EditorForm;
import org.eclipse.hyades.test.ui.editor.form.util.NamedElementSection;
import org.eclipse.hyades.test.ui.editor.form.util.WidgetFactory;
import org.eclipse.hyades.test.ui.internal.action.AddTestComponentAction;
import org.eclipse.hyades.test.ui.internal.editor.form.util.EObjectTreeSection;
import org.eclipse.hyades.test.ui.internal.resources.UiPluginResourceBundle;
import org.eclipse.hyades.test.ui.internal.util.ContextIds;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;

/**
 * @author marcelop
 * @since 0.2.0
 */
public class TestComponentsForm
extends EditorForm implements ISelectionChangedListener
{
	private EObjectTreeSection testComponentsSection;
	private NamedElementSection namedElementSection;
	private AddTestComponentAction addTestComponentAction;
	private boolean firstActivation = true;
	
	/**
	 * Constructor for TestComponentsForm
	 * @param testSuiteEditorExtension
	 * @param widgetFactory
	 */
	public TestComponentsForm(TestSuiteEditorExtension testSuiteEditorExtension, WidgetFactory widgetFactory)
	{
		super(testSuiteEditorExtension, widgetFactory);
		setHeadingText(UiPluginResourceBundle.W_TST_COMPS); 
	}

	/**
	 * @see org.eclipse.update.ui.forms.internal.IForm#dispose()
	 */
	public void dispose()
	{
		testComponentsSection.getTreeViewer().removeSelectionChangedListener(this);
		testComponentsSection.dispose();
		namedElementSection.dispose();
		addTestComponentAction.dispose();
		
		super.dispose();
	}

	/**
	 * Returns the test case suite manipulated by this form.
	 * @return TPFTestSuite
	 */
	protected TPFTestSuite getTestSuite()
	{
		return ((TestSuiteEditorExtension)getBaseEditorExtension()).getTestSuite();		
	}

	/**
	 * @see org.eclipse.hyades.test.ui.internal.editor.form.util.EditorForm#createControl()
	 */
	public Control createControl()
	{
		Control control = super.createControl();
		UiPlugin.getDefault().getWorkbench().getHelpSystem().setHelp(getControl(), UiPlugin.getID() + ContextIds.TESTCOMP_FORM);
		return control;
	}

	/**
	 * @see org.eclipse.hyades.test.ui.internal.editor.form.common.EditorForm#createEditorFormContents(org.eclipse.swt.widgets.Composite)
	 */
	protected void createEditorFormContents(Composite parent)
	{
		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 2;
		gridLayout.makeColumnsEqualWidth = true;
		gridLayout.marginWidth = 10;
		gridLayout.horizontalSpacing = 15;
		parent.setLayout(gridLayout);

		Composite leftColumn = createColumn(parent);
		Composite rightColumn = createColumn(parent);
		
		addTestComponentAction = new AddTestComponentAction();
		testComponentsSection = new EObjectTreeSection(this, Common_TestprofilePackage.eINSTANCE.getTPFTestSuite_TestComponents(), addTestComponentAction);
		registerSection(testComponentsSection);
		testComponentsSection.setHeaderText(UiPluginResourceBundle.W_TST_COMPS); 
		testComponentsSection.setCollapsable(true);
		Control control = testComponentsSection.createControl(leftColumn, getWidgetFactory());
		control.setLayoutData(new GridData(GridData.FILL_BOTH | GridData.VERTICAL_ALIGN_BEGINNING));		
		UiPlugin.getDefault().getWorkbench().getHelpSystem().setHelp(control, UiPlugin.getID() + ContextIds.TESTCOMP_COMP_FORM);										
		UiPlugin.getDefault().getWorkbench().getHelpSystem().setHelp(testComponentsSection.getTreeViewer().getControl(), UiPlugin.getID() + ContextIds.TESTCOMP_TREE_FORM);										
		
		namedElementSection = new NamedElementSection(this);
		namedElementSection.setHeaderText(UiPluginResourceBundle.EDT_GENERAL_INFO); 
		registerSection(namedElementSection);		
		control = namedElementSection.createControl(rightColumn, getWidgetFactory());
		GridData gridData = new GridData(GridData.FILL_HORIZONTAL | GridData.VERTICAL_ALIGN_BEGINNING);
		control.setLayoutData(gridData);
		UiPlugin.getDefault().getWorkbench().getHelpSystem().setHelp(getControl(), UiPlugin.getID() + ContextIds.TESTCOMP_NAME_FORM);										
	}
	
	/**
	 * @see org.eclipse.hyades.test.ui.internal.editor.form.common.EditorForm#load()
	 */
	public void load()
	{
		addTestComponentAction.setTestSuite(getTestSuite());
		
		testComponentsSection.getTreeViewer().removeSelectionChangedListener(this);
		testComponentsSection.setInput(getTestSuite());
		testComponentsSection.getTreeViewer().addSelectionChangedListener(this);
	}
	
	/**
	 * @see org.eclipse.ui.part.ISetSelectionTarget#selectReveal(org.eclipse.jface.viewers.ISelection)
	 */
	public void selectReveal(ISelection selection)
	{
		testComponentsSection.selectReveal(selection);
	}
	
	/**
	 * @see org.eclipse.jface.viewers.ISelectionChangedListener#selectionChanged(org.eclipse.jface.viewers.SelectionChangedEvent)
	 */
	public void selectionChanged(SelectionChangedEvent event)
	{
		if(event.getSource() == testComponentsSection.getTreeViewer())
		{
			IStructuredSelection structuredSelection = (IStructuredSelection)getSelection();
			namedElementSection.setInput(structuredSelection);
		}
	}
	
	public ISelection getSelection()
	{
		return testComponentsSection.getTreeViewer().getSelection();
	}
	
	/**
	 * @see org.eclipse.hyades.test.ui.internal.editor.form.common.EditorForm#activated()
	 */
	public boolean activated()
	{
		if(firstActivation)
		{
			firstActivation = false;
			testComponentsSection.setFocus();
		}
			
		return super.activated();
	}
}