/**********************************************************************
 * Copyright (c) 2005, 2011 IBM Corporation, Intel Corporation.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * $Id: OpenMethodDetailsAction.java,v 1.7 2008-05-20 12:53:46 ialelekov Exp $
 * 
 * Contributors: 
 * IBM - Initial API and implementation
 **********************************************************************/
package org.eclipse.hyades.trace.views.actions.internal;

import java.lang.reflect.InvocationTargetException;

import org.eclipse.core.resources.IResourceStatus;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.Status;
import org.eclipse.hyades.trace.ui.UIPlugin;
import org.eclipse.hyades.trace.views.adapter.internal.ExecutionStatisticViewer2;
import org.eclipse.hyades.trace.views.internal.TraceUIMessages;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;

public class OpenMethodDetailsAction extends OpenTraceViewAction {
	public OpenMethodDetailsAction() {
		super("");
	}

	public OpenMethodDetailsAction(String label) {
		super(label);
	}

	public OpenMethodDetailsAction(String label, ImageDescriptor image) {
		super(label, image);
	}

	public void run() {
		try {
			ExecutionStatisticViewer2 view = ExecutionStatisticViewer2.getActiveViewer();

			// Switch to the appropriate view
			IWorkbenchPage persp = UIPlugin.getActivePage();
			persp.showView(ExecutionStatisticViewer2.VIEW_ID);

			if (view == null) {
				view = (ExecutionStatisticViewer2) persp.showView(ExecutionStatisticViewer2.VIEW_ID);
			}

			final ExecutionStatisticViewer2 fview = view;
			if(fview != null) {
				fview.showMethodDetailsTab();
			}

			// If this exec stats view has not been loaded, then load it and reissue the tab switch
			// command only once the view is loaded. 
			if(fview != null && fview.getCurrentPage() == null) {
				Shell sc = Display.getDefault().getActiveShell();
				new ProgressMonitorDialog(sc).run(true, true, new IRunnableWithProgress() {
					
					public void run(IProgressMonitor monitor) throws InvocationTargetException, InterruptedException {
						monitor.beginTask(TraceUIMessages._276, 1000);
			        	 try {
			        		// Wait at most 5 seconds for the view to open
			        		for(int x = 0; x < 50; x++) {
			        			Thread.sleep(100);
			        			if(fview.getCurrentPage() != null) {
			        				break;
			        			}
			        		}
						} catch (InterruptedException e) { }
			        	Thread.sleep(1000);
			        	monitor.worked(1000);
			        	if(fview != null) fview.showMethodDetailsTab();
						
					}
				});
				
			}

		} catch (Exception e) {
			Status err = new Status(Status.WARNING, ResourcesPlugin.PI_RESOURCES, IResourceStatus.INTERNAL_ERROR, e.toString(), null);

			IWorkbenchWindow window = UIPlugin.getDefault().getWorkbench().getActiveWorkbenchWindow();
			ErrorDialog.openError(window.getShell(), TraceUIMessages._100, "", /* no special message */ err);

			e.printStackTrace();
		}
	}

	public void run(IAction action) {
		run();
	}

	public String getViewID() {
		return ExecutionStatisticViewer2.VIEW_ID;
	}
}
